//=========================DOKUMENTACNI CAST==================================

  /*
   * SLO2VYP.CPP
   *
   * Modul je urcen k chemickym vypoctum, soucast projektu SLO2____.PRJ
   *
   * Martin Slouf, 29.4.1999
   *
   */

//========================POTREBNE INCLUDE SOUBORY============================

  /*
   * systemove .H soubory #include <stdio.h>
   *
   * vlastni .H soubory #include "vzor.h"
   *
   */

#include <ctype.h>
#include <stdio.h>
#include <math.h>
#include "slo2vyp.h"
#include "slo2tab.h"

//===================DEFINICE GLOBALNICH IMPORTOVANYCH OBJEKTU================

  //zadne by byt nemely - vse by melo byt v .H souborech

  /*
   * jde o importovane objekty (fce nebo promenne) - jiz musi byt v nejakem
   * modulu definovany (v casti globalnich definic)
   *
   * zde se objevi jako extern deklarace
   *
   */

//=================DEFINICE GLOBALNICH EXPORTOVANYCH PROMENNYCH===============

  /*
   * jde o globalni promenne, ktere maji byt sdileny s jinymi moduly, zde
   * nejsou definovany jako extern - jde o jejich "domaci" modul
   *
   * ve sdilenych modulech se objevi za pomoci .H souboru prislusejiciho
   * k tomuto modulu (v tomto .H souboru budou uvedeny jako extern deklarace)
   *
   */

//=======================LOKALNI DEFINICE PRO TENTO MODUL=====================

  /*
   * lokalni definice pouze pro tento modul (soubor), pokud by tyto definice
   * byly prilis rozsahle -> specialni .H soubor
   *
   */

  /*
   * 1. symbolicke konstanty a makra s parametry
   *
   */

#define PI 3.14159265358979323846
  /* Rudolfovo cislo PI na 21 desetinnych mist */
#define st_na_rad(uhel) uhel = uhel * PI / 180
  /* prevede stupne na radiany */
#define sqr(x) ((x) * (x))
  /* druha mocnina */

  /*
   * 2. typy - zasadne pomoci typedef
   *
   */

  /*
   * 3. promenne - jde o globalni promenne pro tento modul - v pametove tride
   *    static
   *
   */

  /*
   * 4. uplne funkcni prototypy lokalnich (=pro tento modul) funkci, tj.
   * v pametove tride static
   *
   */

  /*
   * 5. fce main - pokud jde o hlavni soubor projektu
   *
   */

//========================DEFINICE GLOBALNICH FUNKCI==========================

  /*
   * jde o funkce, ktere jsou exportem modulu, jejich uplne funkcni prototypy
   * jsou v prislusnem .H souboru (obdobne jako u globalnich promennych, ani
   * zde neni uvedena zadna pametova trida, ale v prislusnem .H souboru jsou
   * deklarovany jako extern)
   *
   */

  /*
   * 1. Vypocet molarni hmotnosti
   *
   * 2. Vypocet objemu
   *
   * 3. Vypocet krystalograficke hustoty
   *
   */

  /********************************
   *                              *
   * 1. Vypocet molarni hmotnosti *
   *                              *
   ********************************/

//--------------------------VYPOCET_MR----------------------------------------
double vypocet_mr(const char s[])
  /* vraci mr vzorcove jednotky - vycet atomu v symetricky nezavisle casti
   * bunky - ulozene v s[]
   * vraci 0.0 v pripade chyby
   */
{
  ZNACKA znacka; /* znacka atomu v s[] */
  int index; /* index u atomu udavajici jeho pocet ve vzorcove jednotce */
    /* priklad zadani:
     * H2O -> atom vodiku (H) * 2 + atom kysliku (O) * 1
     * tudiz -> znacka nabude postupne hodnoty "H" a "O"
     *          index nabude postupne hodnoty "2" a "1"
     * ciste pro jistotu si drzim 5 bytu pameti pro ctyrciferne cislo
     */
  P_PRVEK atom; /* pomoci tohoto ukazatele zjistuji udaje o atomu */
  int c; /* kvuli pristupu k jednotlivym znakum retezce */
  int i = 0, j; /* promenna cyklu */
  double mezivysledek = 0.0;

  while (s[i] != '\0') {

    j = 0; znacka[0] = '\0';
      /* nacitam znacku dokud nejsem nakonci, je to pismeno a neni to delsi
       * nez dva znaky
       */
    while (s[i] != '\0' && isalpha(s[i]) && j < 2) {
      znacka[j++] = s[i++];
      if (isupper(s[i]))
        break;
    }
    znacka[j] = '\0';

    if ((atom = je_atom(znacka)) == NULL)
      return(0.0); /* chybne zadany retez - vraci se neuspech */

    index = 0;
    while (s[i] != '\0' && isdigit(s[i]))
      index = index * 10 + s[i++] - '0';
        /* prevod ctenych cislic na dekadicke cislo */

    if (index == 0) /* za atomem nebyla jedina cislice */
      index = 1;

    /* nyni nasleduje test na pripad, kdy znak s[i] neni alphanumericky znak
     * tj. ani cislice, ani pismeno
     * muze jit o tildu, tecku, atd.
     * vysledek -> chyba v datech
     */
    if (!isalpha(s[i]) && !isdigit(s[i]) && s[i] != '\0')
      return(0.0); /* chybne zadany retez - vraci se neuspech */

    mezivysledek += (index * atom->mr);
  }

  return(mezivysledek);
}

  /*********************
   *                   *
   * 2. Vypocet objemu *
   *                   *
   *********************/

//----------------------------KUBICKA-----------------------------------------
double kubic()
{
  double a = 0.0;

  printf("Zadej stranu krychle:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej stranu krychle:\n");
  }

  return(a * a * a);
}

//----------------------------TETRAGONALNI------------------------------------
double tetragonal()
{
  double a, c;

  a = c = 0.0;

  printf("Zadej 1. stranu:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej 1. stranu:\n");
  }

  printf("Zadej 2. stranu:\n");
  while (scanf("%lf", &c) != 1) {
    fflush(stdin);
    printf("Zadej 2. stranu:\n");
  }

  return(a * a * c);
}

//----------------------------ORTOROMBICKA------------------------------------
double ortorombic()
{
  double a, b, c;

  a = b = c = 0;

  printf("Zadej 1. stranu:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej 1. stranu:\n");
  }

  printf("Zadej 2. stranu:\n");
  while (scanf("%lf", &b) != 1) {
    fflush(stdin);
    printf("Zadej 2. stranu:\n");
  }

  printf("Zadej 3. stranu:\n");
  while (scanf("%lf", &c) != 1) {
    fflush(stdin);
    printf("Zadej 3. stranu:\n");
  }

  return(a * b * c);
}

//----------------------------HEXAGONALNI-------------------------------------
double hexagonal()
{
  double a, c;
  double gama;

  a = c = 0.0;
  gama = 120.0;

  printf("Zadej 1. stranu:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej 1. stranu:\n");
  }

  printf("Zadej 2. stranu:\n");
  while (scanf("%lf", &c) != 1) {
    fflush(stdin);
    printf("Zadej 2. stranu:\n");
  }

  st_na_rad(gama);

  return(a * a * c * sqrt(1 - sqr(cos(gama))));
}

//----------------------------ROMBOEDRICKA------------------------------------
double romboedric()
{
  double a = 0.0;
  double alfa = 0.0;

  printf("Zadej stranu:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej stranu:\n");
  }

  printf("Zadej uhel:\n");
  while (scanf("%lf", &alfa) != 1) {
    fflush(stdin);
    printf("Zadej uhel:\n");
  }

  st_na_rad(alfa);

  return(a * a * a * sqrt(1 - 3 * sqr(cos(alfa)) + 2 * cos(alfa) * cos(alfa) * cos(alfa)));
}
//-------------------------MONOKLINICKA---------------------------------------
double monoklinic()
{
  double a, b, c;
  double beta;

  a = b = c = beta = 0.0;

  printf("Zadej 1. stranu:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej 1. starnu:\n");
  }

  printf("Zadej 2. stranu:\n");
  while (scanf("%lf", &b) != 1) {
    fflush(stdin);
    printf("Zadej 2. starnu:\n");
  }

  printf("Zadej 3. stranu:\n");
  while (scanf("%lf", &c) != 1) {
    fflush(stdin);
    printf("Zadej 3. starnu:\n");
  }

  printf("Zadej uhel pri vyznacnbe ose (beta):\n");
  while (scanf("%lf", &beta) != 1) {
    fflush(stdin);
    printf("Zadej uhel pri vyznacnbe ose (beta):\n");
  }

  st_na_rad(beta);

  return(a * b * c * sqrt(1 - sqr(cos(beta))));
}

//-----------------------------TRIKLINICKA------------------------------------
double triklinic()
{
  double a, b, c;
  double alfa, beta, gama;

  a = b = c = alfa = beta = gama = 0.0;

  printf("Zadej 1. stranu:\n");
  while (scanf("%lf", &a) != 1) {
    fflush(stdin);
    printf("Zadej 1. starnu:\n");
  }

  printf("Zadej 2. stranu:\n");
  while (scanf("%lf", &b) != 1) {
    fflush(stdin);
    printf("Zadej 2. starnu:\n");
  }

  printf("Zadej 3. stranu:\n");
  while (scanf("%lf", &c) != 1) {
    fflush(stdin);
    printf("Zadej 3. starnu:\n");
  }

  printf("Zadej uhel alfa:\n");
  while (scanf("%lf", &alfa) != 1) {
    fflush(stdin);
    printf("Zadej uhel alfa:\n");
  }

  st_na_rad(alfa);

  printf("Zadej uhel beta:\n");
  while (scanf("%lf", &beta) != 1) {
    fflush(stdin);
    printf("Zadej uhel beta:\n");
  }

  st_na_rad(beta);

  printf("Zadej uhel gama:\n");
  while (scanf("%lf", &gama) != 1) {
    fflush(stdin);
    printf("Zadej uhel gama:\n");
  }

  st_na_rad(gama);

  return(a * b * c * sqrt(1 - sqr(cos(alfa)) - sqr(cos(beta)) - sqr(cos(gama)) + 2 * cos(alfa) * cos(beta) * cos(gama)));
}

  /***************************************
   *                                     *
   * 3. Vypocet krystalograficke hustoty *
   *                                     *
   ***************************************/

//-------------------------------VYPOCET_DX-----------------------------------
double vypocet_dx(double mr, double objem)
{
  return((mr * MU) / (objem * exp(-27 * log(10))));
}

//========================DEFINICE LOKALNICH FUNKCI===========================

  /*
   * definice funkci pro tento modul (pametova trida static)
   *
   * jejich funkcni prototypy - viz vyse
   *
   */