#ifndef _DATA_H
#define _DATA_H

//==========================DOKUMENTACNI CAST=================================

  /*
   * DATA.H, verze 1.0
   *
   * Hlavickovy soubor k modulu SLO3DATA.CPP, soucast projektu SLO3____.PRJ
   * Obsahuje fce a typy pro obsluhu dat
   *
   * Martin Slouf, 22.5.1999
   *
   */

//========================DEFINICE SYMBOLICKYCH KONSTANT======================

  /*
   * ktere budou vyuzivany i v jinych modulech
   *
   */

#define MAXDELKASLOVA          20
#define MAXDELKAVETNEHOCLENU   40
#define POCETDRUHUVETNYCHCLENU  7
  /* sedmy druh je NEZNAMY -> slouzi k identifikaci, pokud neznam vetny clen,
   * ci pokud chci priradit neurcitou hodnotu
   */
#define MAXPOCETJMENOVEKVEVC    5
  /* maximalni pocet jmenovek ve vetnem clenu */

//=======================DEFINICE MAKER S PARAMETRY===========================

  /*
   * ktere budou vyuzivany i v jinych modulech
   *
   */

#define je_fronta_slov_prazdna(f) (((f).p_celo == (f).p_volny) ? 1 : 0)
  /* pro frontu slov */
#define je_fronta_vc_prazdna(f) (((f).p_celo == (f).p_volny) ? 1 : 0)
  /* pro frontu vetnych clenu */

//========================DEFINICE GLOBALNICH TYPU============================

  /*
   * zasadne pomoci typedef
   *
   */

/* ---------------------------------TYP SLOVO------------------------------ */

/* druh slova */
typedef enum {
  PODJ = 1, PRIDJ, ZAJMENO, CISLOVKA, SLOVESO, PRISLOVCE, PREDLOZKA, SPOJKA,
  CASTICE, CITOSLOVCE, DRUHSLOVASENEPOUZIVA
} DRUHSLOVA;

/* typ vetneho clenu - pozor! - odlisuji typ a druh vetneho clenu */
typedef enum {
  SAMOSTATNY = 1, NESAMOSTATNY, NENI
} TYPVETNEHOCLENU;

/* pad */
typedef enum {
  NOMINATIV = 1, GENITIV, DATIV, AKUZATIV, VOKATIV, LOKAL, INSTRUMENTAL,
  PADSENEPOUZIVA
} PAD;

/* jmenny rod (masculinum, feminimum, neutrum) */
typedef enum {
  M = 1, F, N, RODSENEPOUZIVA
} ROD;

/* cislo (singular, plural) */
typedef enum {
  SG = 1, PL, CISLOSENEPOUZIVA
} CISLO;

/* tvar slovesa */
typedef enum {
  URCITY = 1, NEURCITY_PRICESTI, NEURCITY_INFINITIV, TVARSENEPOUZIVA
} TVAR;

/* osoba */
typedef enum {
  PRVNI = 1, DRUHA, TRETI, OSOBASENEPOUZIVA
} OSOBA;

/* vlastnosti slova */
typedef struct {
  /* sklonovaci typy */
  PAD pad;
  ROD rod;
  CISLO cislo_jmena;
  /* sloveso - casovaci typ */
  TVAR tvar;
  OSOBA osoba;
  CISLO cislo_slovesa;
  ROD jmenny_rod;
  /* neohebne */
  PAD predlozka;
} VLASTNOSTISLOVA;

/* TYP SLOVO */
typedef struct {
  char jmenovka[MAXDELKASLOVA];
  DRUHSLOVA druh;
  TYPVETNEHOCLENU typ_vetneho_clenu;
  VLASTNOSTISLOVA vlastnosti;
  int poradi;
} SLOVO, *P_SLOVO;

/* -------------------------TYP FRONTA SLOV-------------------------------- */

/* prvek fronty slov */
typedef struct pfs{
  struct pfs *p_dalsi;
  SLOVO slovo;
} PFS, *P_PFS;

/* TYP FRONTA SLOV */
typedef struct {
  P_PFS p_celo;
  P_PFS p_volny;
} FRONTASLOV;

/* ---------------------------TYP VETNYCLEN-------------------------------- */

/* druh vetneho clenu */
typedef enum {
  PRISUDEK = 1, PODMET, PRIVLASTEK, PREDMET, PRISL_URCENI, DOPLNEK, NEZNAMY
} DRUHVETNEHOCLENU;

/* boolean */
typedef enum {
  NE = 0, ANO
} BOOLEAN;

/* vlastnosti vetneho clenu */
typedef struct {
  BOOLEAN rozvity;
  BOOLEAN nekolikanasobny;
} VLASTNOSTIVETNEHOCLENU;

/* TYP VETNY CLEN */
typedef struct {
  char jmenovka[MAXDELKAVETNEHOCLENU];
  DRUHVETNEHOCLENU druh[POCETDRUHUVETNYCHCLENU];
  DRUHSLOVA druh_slova[MAXPOCETJMENOVEKVEVC];
  int poradi_jmenovky_ve_vete[MAXPOCETJMENOVEKVEVC];
  struct vetnyclen *p_nadr;
  struct vetnyclen *p_podr;
  VLASTNOSTIVETNEHOCLENU vlastnosti;
  int poradi;
} VETNYCLEN, *P_VETNYCLEN;

/* --------------------------TYP FRONTA VETNYCH CLENU---------------------- */

/* prvek fronty vetnych clenu */
typedef struct pfvc {
  struct pfvc *p_dalsi;
  VETNYCLEN vetnyclen;
} PFVC, *P_PFVC;

/* TYP FRONTA VETNYCH CLENU */
typedef struct {
  P_PFVC p_celo;
  P_PFVC p_volny;
} FRONTAVETNYCHCLENU;

//==========DEKLARACE GLOBALNICH PROMENNYCH PRISLUSNEHO .CPP MODULU===========

  /*
   * !!! deklarace, nikoli definice !!!
   *
   * jsou predmetem exportu .CPP modulu - zde uvedeny jako extern deklarace
   *
   */

//===============UPLNE FUNKCNI PROTOTYPY PRISLUSNEHO .CPP MODULU==============

  /*
   * jsou predmetem exportu .CPP modulu - zde uvedeny jako extern deklarace
   *
   */

/* SPRAVA FRONTY SLOV */
extern int vytvor_frontu_slov(FRONTASLOV *fs);
  /* vytvori frontu *fs v pameti
   * vraci 1, pokud je to Ok, 0 pri nedostatku pameti
   */
extern int vloz_na_konec_fronty_slov(FRONTASLOV *fs, P_SLOVO p_s);
  /* na konec fronty (do prvku volny) vlozi hodnotu p_s
   * vraci 1, je-li operace uspesne provedena
   */
extern int odeber_celo_fronty_slov(FRONTASLOV *fs, SLOVO *s);
  /* odebere prvni prvek fronty (prvek, kam ukazuje celo)
   * vraci 1, je-li vse Ok
   * v promenne s vraci hodnotu prvku, je dobre si uvedomit, ze bych mohl
   * pouzit i deklaraci P_SLOVO p_s, ale uvadim radeji SLOVO *s, aby bylo
   * patrne, ze vetsinou prvek odebiram do jednoho statickeho prvku s a
   * do fronty ho posilam pomoci dereference
   */
extern void uvolni_frontu_slov(FRONTASLOV *fs);
  /* uvolni pamet alokovanou frontou slov fs */
extern P_PFS get_prvek_z_fs_s_poradim(FRONTASLOV *fs, int poradi);
  /* vrati pointer na prvek na dane pozici ve fronte slov fs*/

/* SPRAVA SEZNAMU VETNYCH CLENU */
extern int vytvor_frontu_vc(FRONTAVETNYCHCLENU *fvc);
extern int vloz_na_konec_fronty_vc(FRONTAVETNYCHCLENU *fvc, P_VETNYCLEN p_vc);
extern int odeber_celo_fronty_vc(FRONTAVETNYCHCLENU *fvc, VETNYCLEN *vc);
extern void uvolni_frontu_vc(FRONTAVETNYCHCLENU *fvc);

#endif